import os
import platform
from utils import get_bool_envvar

#############################################################################
################ Environment Variables from Jenkins #########################
#############################################################################

# Whether or not to pause the CI. Useful when needing to use
# a static MAAS instance.
PAUSE_CI = get_bool_envvar('PAUSE_CI', False)

# Whether or not to enable MAAS debug logging.
DEBUG = get_bool_envvar('DEBUG', False)

# Series to install on deployed nodes.
NODE_SERIES = os.environ.get('NODE_SERIES', 'bionic')

# Nodes (and architectures)
USE_PPC_NODES = get_bool_envvar('USE_PPC_NODES', False)
USE_ARM64_NODES = get_bool_envvar('USE_ARM64_NODES', False)
# Whether or not to use EA's systems.
USE_DELL_R630_SYSTEMS = get_bool_envvar('USE_DELL_R630_SYSTEMS', False)

# Images
TEST_RHEL = get_bool_envvar('TEST_RHEL', False)
TEST_CENTOS = get_bool_envvar('TEST_CENTOS', False)
TEST_WINDOWS = get_bool_envvar('TEST_WINDOWS', False)
TEST_CUSTOM_IMAGES = get_bool_envvar('TEST_CUSTOM_IMAGES', False)
CUSTOM_IMAGES_SOURCE = os.environ.get(
                           'CUSTOM_IMAGES_SOURCE',
                           'http://10.245.136.7/images/streams/v1/index.json')

# Performance
TEST_PERFORMANCE = get_bool_envvar('TEST_PERFORMANCE', False)

# Which slave to use
SLAVE_NODE = os.environ.get('SLAVE_NODE', 'ci-lab')

##################### Deprecated variables that need updates ################
# Whether or not the ARM nodes should be used
# in the test.
# TODO: These systems are no longer used/supported
DO_NOT_USE_ARM_NODES = bool(
    os.environ.get('DO_NOT_USE_ARM_NODES', True))
USE_ARM_NODES = not DO_NOT_USE_ARM_NODES

# Whether or not the cluster controller nodes in different network
# should be used.
USE_CC_NODES = bool(os.environ.get('USE_CC_NODES', False))
##################### Deprecated variables that need updates ################


#############################################################################
############ Global variables that define MAAS configuration     ############
#############################################################################

# MAAS user account
ADMIN_USER = "admin"
PASSWORD = "test"

# How to configure DHCP
if SLAVE_NODE in ['orobas', 'ci-lab-ipv6']:
    REGION_CONFIG = {
        "ip": "[2001:67c:1562:800a::6]",
        "router_ip": "fe80::1",
        "management": 2,
        "ip_range_low": "2001:67c:1562:800a:dddd::",
        "ip_range_high": "2001:67c:1562:800a:ddde::",
        # add cidr as it is now default in new networking configs.
        "cidr": "2001:67c:1562:800a::/64",
        # bmc ips:
        "bmc_start_ip": "2001:67c:1562:800a:cccc::",
        "bmc_end_ip": "2001:67c:1562:800a:cccd::",
        # DNS config:
        # dnssec is now enabled by default, so disable it to avoid errors from the
        # forwarder server.
        # See StackOverflow question "ns got insecure response, parent indicates
        # it should be secure": http://bit.ly/1pe8Hkk
        "upstream_dns": "10.245.88.1",
        "dnssec_validation": "no",
        # HTTP proxy used to access the external world.
        # TODO: This needs updating and obtained from Jenkins.
        "http_proxy": "http://squid.internal:3128",
        # expected internal proxy DNS domain
        "internal_proxy": "http://2001-67c-1562-800a----64.maas-internal:8000/",
    }

else:
    REGION_CONFIG = {
        "ip": "10.245.136.6",
        "interface": "eth1",
        "subnet_mask": "255.255.248.0",
        "broadcast_ip": "10.245.143.255",
        "router_ip": "10.245.136.1",
        "management": 2,
        "ip_range_low": "10.245.136.10",
        "ip_range_high": "10.245.136.200",
        # add cidr as it is now default in new networking configs.
        "cidr": "10.245.136.0/21",
        # bmc ips:
        "bmc_start_ip": "10.245.143.100",
        "bmc_end_ip": "10.245.143.200",
        # DNS config:
        # dnssec is now enabled by default, so disable it to avoid errors from the
        # forwarder server.
        # See StackOverflow question "ns got insecure response, parent indicates
        # it should be secure": http://bit.ly/1pe8Hkk
        "upstream_dns": "10.245.136.1",
        "dnssec_validation": "no",
        # HTTP proxy used to access the external world.
        # TODO: This needs updating and obtained from Jenkins.
        "http_proxy": "http://squid.internal:3128",
        # expected internal proxy DNS domain
        "internal_proxy": "http://10-245-136-0--21.maas-internal:8000/",
    }

MAAS_URL = "http://%s:5240/MAAS" % REGION_CONFIG['ip']
MAIN_ARCHIVE = "http://us.archive.ubuntu.com/ubuntu"
PORTS_ARCHIVE = "http://ports.ubuntu.com/ubuntu-ports"
USER_DATA_URL = "http://%s:5240/MAAS/metadata/enlist/latest/user-data" % REGION_CONFIG['ip']
ENLIST_PRESEED_URL = (
    "http://%s:5240/MAAS/metadata/latest/enlist-preseed/?op="
    "get_enlist_preseed" % REGION_CONFIG['ip'])

# If not defined, we will always assume we will test CentOS.
RHEL_IMAGE = "/home/ubuntu/rhel7-amd64-root-tgz"
WINDOWS_IMAGE = "/home/ubuntu/windows-win2012hvr2-amd64-root-dd"


##################### Deprecated variables that need updates ################
CLUSTER_CONTROLLER_IP = REGION_CONFIG['ip']
SQUID_DEB_PROXY_URL = 'http://10.55.32.185:8000'
# Whether or not set the http proxy. Currently only working on raring
DO_NOT_SET_PROXY = bool("raring" not in platform.linux_distribution())
##################### Deprecated variables that need updates ################


#############################################################################
################ Global variables for machines in MAAS $$$###################
#############################################################################

# Declares the machines that will be added as devices to prevent MAAS from
# hanging out IP addresses that would conflict.
DEVICES_CI_LAB = [
    {
        "hostname": "lenovo-RD230-01",
        "mac": "00:25:9C:2F:C7:9E",
        "ip": "10.245.136.5",
    },
    # Jenkins master running on on old jenkins slave.
    # Currently used as the new lander
    {
        "hostname": "lenovo-RD230-01-jenkins-master",
        "mac": "00:e0:81:dd:d1:cf",
        "ip": "10.245.136.2",
    },
    # BMC IP is (originally used as a testing system)
    {
        "hostname": "lenovo-RD230-07-jenkins-slave",
        "mac": "00:e0:81:dd:d1:1b",
        "ip": "10.245.136.3",
    },
    {
        "hostname": "HP-DL360-Gen9",
        "mac": "94:18:82:76:C0:64",
        "ip": "10.245.136.7",
    },
    # This is the new master jenkins. This is where KPIs run.
    {
        "hostname": "jenkins",
        "mac": "00:16:3e:a7:a1:a4",
        "ip": "10.245.136.4",
    },
    # ltrager XXX 2019-03-27 - Temporarily removed from CI for ESXi vCenter integration.
    {
        "hostname": "esxi-01",
        "mac": "ec:b1:d7:7f:ef:34",
        "ip": "10.245.136.201",
    },
    {
        "hostname": "vcenter",
        "mac": "00:0c:29:48:e8:67",
        "ip": "10.245.136.202",
    },
    # This container runs the Prometheus/Grafana for collecting MAAS
    # performance metrics from CI runs.
    {
        "hostname": "maas-performance",
        "mac": "00:16:3e:00:7c:18",
        "ip": "10.245.136.203",
    },
]

DEVICES_OROBAS = [
    #{
    #    "hostname": "jenkins-master-orobas",
    #    "mac": "00:16:3e:98:ef:88",
    #    "ip": "10.246.88.4",
    #},
    #{
    #    "hostname": "lihko-bmc",
    #    "mac": "",
    #    "ip": "10.246.88.15",
    #},
    {
        "hostname": "orobas",
        "mac": "e4:11:5b:b8:36:a8",
        "ip": "2001:67c:1562:800a::3",
    },
]

OROBAS_SYSTEMS = {
    'power_user': 'maasadmin',
    'power_pass': 'maasteam',
    'power_driver': 'LAN_2_0',
    'console': '',
    'machines': {
        # likho
        "E4:11:5B:B9:D1:12": "10.246.88.15",
    }
}

# Lenovo machines
POWER_USER = "root"
POWER_PASS = "ubuntu"
LENOVO_SYSTEMS = {
    'power_user': 'root',
    'power_pass': 'ubuntu',
    'power_driver': 'LAN',
    'console': '',
    'machines': {
        # lenovo-RD230-03, MAAS-03
        "00:E0:81:DD:D5:99": "10.245.143.103",
        # lenovo-RD230-04, MAAS-04
        #"00:E0:81:DD:D1:0B": "10.245.143.104", # TODO: Investigate. Seems broken
        # lenovo-RD230-06, MAAS-06
        "00:E0:81:D1:B1:47": "10.245.143.106",
        # lenovo-RD230-08, MAAS-08
        #"00:E0:81:DD:D1:2B": "10.245.143.108",
        # lenovo-RD230-10, MAAS-10
        "00:E0:81:DC:38:6D": "10.245.143.110",
        # lenovo-RD230-01, MAAS-01
        #"": "10.245.143.101" # TODO: Previously used for Jenkins Master.
        # lenovo-RD230-07, MAAS-07
        #"00:E0:81:DD:D1:1B": "10.245.143.107", # XXX: Used for Jenkins Slave.
    }
}

# TODO: mapping is outdated.
ARM_LAB = {
    'fc:2f:40:d8:fb:1a': '192.168.21.50',
    'fc:2f:40:fa:2f:9a': '192.168.21.51',
    'fc:2f:40:ec:73:42': '192.168.21.52',
    'fc:2f:40:d5:87:ce': '192.168.21.53'
}

# PPC64LE
PPC_SYSTEMS = {
    'power_user': '',
    'power_pass': 'admin',
    'power_driver': 'LAN_2_0',
    'console': '',
    'machines': {
        '98:BE:94:02:6A:C8': '10.245.143.113',
    }
}

# ARM64
GIGABYTE_SYSTEMS = {
    'power_user': 'admin',
    'power_pass': 'password',
    'console': '',
    'machines': {
        '1c:1b:0d:0d:52:7c': '10.245.143.114',
    }
}

# HP Gen9
GEN9_SYSTEMS = {
    'power_user': 'Administrator',
    'power_pass': 'insecure',
    'console': '',
    'machines': {
        "94:18:82:7a:72:44": "10.245.143.116", # EFI
        "94:18:82:7a:72:64": "10.245.143.117", # EFI
        "94:18:82:7a:82:c8": "10.245.143.118", # EFI
        "94:18:82:7a:72:80": "10.245.143.119", # EFI
    }
}

# Landscape hardware
LANDSCAPE_SYSTEMS = {
    'power_user': 'landscape',
    'power_pass': 'insecure',
    'console': '',
    'machines': {
        "2c:59:e5:4a:be:cc": "10.245.143.122", # amco
        "2c:59:e5:3b:c0:e4": "10.245.143.123", # barley
        "ec:b1:d7:80:00:08": "10.245.143.124", # lapras XXX 2018-08-01: seems broken # EFI
        ### "ec:b1:d7:7f:ef:94": "10.245.143.125", # pidgey XXX 2018-07-04: seems broken
        "2c:59:e5:3b:11:28": "10.245.143.126", # squier
        # ltrager XXX 2019-03-27 - Temporarily removed from CI for ESXi vCenter integration.
        # "ec:b1:d7:7f:ef:34": "10.245.143.127", # stunky XXX 2018-08-01 seems broken # EFI
    }
}

# Dell Hardware for CentOS / Windows CI testing
DELL_R630_SYSTEMS = {
    'power_user': 'root',
    'power_pass': 'calvin',
    'console': 'console=ttyS0,115200',
    'machines': {
        "f4:8e:38:d0:0e:d2": "10.245.143.120", # natasha
        "f4:8e:38:d0:0b:54": "10.245.143.121", # opelt
    }
}

CLUSTER_CONTROLLER_NODES = None

if SLAVE_NODE in ['orobas', 'ci-lab-ipv6']:
    SLAVES = DEVICES_OROBAS
    MAIN_SYSTEMS = OROBAS_SYSTEMS
    # Whether or not to use extra amd64 systems. Default to True
    USE_GEN9_NODES = False
    USE_LANDSCAPE_NODES = False
else:
    SLAVES = DEVICES_CI_LAB
    MAIN_SYSTEMS = LENOVO_SYSTEMS
    # Whether or not to use extra amd64 systems. Default to True
    USE_GEN9_NODES = True
    USE_LANDSCAPE_NODES = True

if USE_CC_NODES:
    MAIN_SYSTEMS.update(CLUSTER_CONTROLLER_NODES)

#############################################################################
################ Global variables for configuration not in maas #############
#############################################################################

# This is old configuration that's not tested on newer version of MAAS
# that support setting forwarder and changing DNSSec validation.
LAB_DNS_CONFIG = """
options {
    directory "/var/cache/bind";
    forwarders {
    10.245.136.1;
    };
    allow-query-cache {any;};
    dnssec-enable no;
    dnssec-validation no;
    auth-nxdomain no;    # conform to RFC1035
};
"""

JUJU_CONFIG = """
default: maas
environments:
  maas:
    type: maas
    maas-server: '%s'
    maas-oauth: '%s'
    bootstrap-timeout: 18000
    admin-secret: 'nothing'
    default-series: %s
    logging-config: <root>=TRACE
    disable-network-management: true
"""

TEST_JUJU = get_bool_envvar('TEST_JUJU', False)

# User data to deploy a machine and configure libvirt for a pod.
POD_USER_DATA = """#!/bin/bash
apt-get install bridge-utils qemu-kvm libvirt-bin -y

# Handle installing extra packages for arm64
if [ `uname -p` == "aarch64" ]; then
        apt-get install qemu-efi -y
elif [ `uname -p` == "ppc64le" ]; then
        ppc64_cpu --smt=off
fi

virsh net-destroy default  
virsh net-undefine default

cat <<EOF > net-maas.xml
<network>  
	<name>maas</name>  
	<forward mode="bridge" />  
	<bridge name="br0" />  
</network>
EOF

virsh net-define net-maas.xml
virsh net-autostart maas
virsh net-start maas
rm net-maas.xml

virsh pool-define-as default dir - - - - "/var/lib/libvirt/images"  
virsh pool-autostart default  
virsh pool-start default

echo -e "ubuntu\nubuntu" | passwd ubuntu
sed -i 's/PasswordAuthentication no/PasswordAuthentication yes/g' /etc/ssh/sshd_config
service ssh restart
"""
